unit OpenWireBasicStreamTypes;

interface
uses
  OpenWirePins, classes;

type
  IOWIntegerStream = interface(IOWStream)
    ['{92F6B47E-B46E-49BB-9A8E-676D3C2F2E66}']
    procedure SendIntegerData ( Data : Integer ); stdcall;
  end;

  IOWFloatStream = interface(IOWStream)      
    ['{67F6997B-7EB4-4E2F-8320-4A512B5F2BC7}']
    procedure SendFloatData ( Data : Single ); stdcall;
  end;

  IOWRealStream = interface(IOWStream)
    ['{208F9564-3C20-4C97-BB77-2FC5AE6E4194}']
    procedure SendRealData ( Data : Real ); stdcall;
  end;

type TOWSendFloatEvent = procedure ( Data : Single; CustomData : Pointer ) of object;
type TOWFloatPinNotificationEvent = function (Handler : IOWFloatStream; Operation : Integer; Data : Pointer; State : TOWNotificationState ) : TOWNotificationResults of object;

type TOWSendIntegerEvent = procedure ( Data : Integer; CustomData : Pointer ) of object;
type TOWIntegerPinNotificationEvent = function (Handler : IOWIntegerStream; Operation : Integer; Data : Pointer; State : TOWNotificationState ) : TOWNotificationResults of object;

type TOWSendRealEvent = procedure ( Data : Real; CustomData : Pointer ) of object;
type TOWRealPinNotificationEvent = function (Handler : IOWRealStream; Operation : Integer; Data : Pointer; State : TOWNotificationState ) : TOWNotificationResults of object;

  TOWFloatSourcePin = class(TOWSourcePin )

  protected
    PinNotificationEvent        : TOWFloatPinNotificationEvent;

  public
    constructor Create( AOwner: TComponent; APinNotificationEvent : TOWFloatPinNotificationEvent );

  protected
  public
    function Notification( Handler : IOWStream; Operation : Integer; Data : Pointer; State : TOWNotificationState ) : TOWNotificationResults; virtual;
  public
  end;

  TOWFloatSinkPin = class( TOWSinkPin, IOWFloatStream )

  protected
    FCustomData : Pointer;
    SendFloatEvent : TOWSendFloatEvent;

  public
    procedure SendFloatData ( Data : Single ); stdcall;

  protected
  public
    constructor Create( AOwner: TComponent; ASendFloatEvent : TOWSendFloatEvent; ACustomData : Pointer );

  end;
  
  TOWIntegerSourcePin = class(TOWSourcePin )

  protected
    PinNotificationEvent        : TOWIntegerPinNotificationEvent;

  public
    constructor Create( AOwner: TComponent; APinNotificationEvent : TOWIntegerPinNotificationEvent );

  protected
  public
    function Notification( Handler : IOWStream; Operation : Integer; Data : Pointer; State : TOWNotificationState ) : TOWNotificationResults; virtual;
  public
  end;

  TOWIntegerSinkPin = class( TOWSinkPin, IOWIntegerStream )

  protected
    FCustomData : Pointer;
    SendIntegerEvent : TOWSendIntegerEvent;

  public
    procedure SendIntegerData ( Data : Integer ); stdcall;

  protected
  public
    constructor Create( AOwner: TComponent; ASendIntegerEvent : TOWSendIntegerEvent; ACustomData : Pointer );

  end;

  TOWRealSourcePin = class(TOWSourcePin )

  protected
    PinNotificationEvent        : TOWRealPinNotificationEvent;

  public
    constructor Create( AOwner: TComponent; APinNotificationEvent : TOWRealPinNotificationEvent );

  protected
  public
    function Notification( Handler : IOWStream; Operation : Integer; Data : Pointer; State : TOWNotificationState ) : TOWNotificationResults; virtual;
  public
  end;

  TOWRealSinkPin = class( TOWSinkPin, IOWRealStream )

  protected
    FCustomData : Pointer;
    SendRealEvent : TOWSendRealEvent;

  public
    procedure SendRealData ( Data : Real ); stdcall;

  protected
  public
    constructor Create( AOwner: TComponent; ASendRealEvent : TOWSendRealEvent; ACustomData : Pointer );

  end;
  
procedure Register;

implementation

constructor TOWFloatSourcePin.Create( AOwner: TComponent; APinNotificationEvent : TOWFloatPinNotificationEvent );
begin
  inherited Create( AOwner );
  PinNotificationEvent := APinNotificationEvent;

  AddType( IOWFloatStream, Notification );
end;
//---------------------------------------------------------------------------
function TOWFloatSourcePin.Notification( Handler : IOWStream; Operation : Integer; Data : Pointer; State : TOWNotificationState ) : TOWNotificationResults;
var
  Interf : IOWFloatStream;

begin
  if( Assigned( PinNotificationEvent )) then
    if( Handler.QueryInterface(IOWFloatStream,Interf) = 0 ) then
       begin
       Result := PinNotificationEvent( Interf, Operation, Data, State );
       Exit;
       end;

  Result := [];
end;
//---------------------------------------------------------------------------
constructor TOWFloatSinkPin.Create( AOwner: TComponent; ASendFloatEvent : TOWSendFloatEvent; ACustomData : Pointer );
begin
  inherited Create( AOwner );
  SendFloatEvent := ASendFloatEvent;
  FCustomData := ACustomData;
end;
//---------------------------------------------------------------------------
procedure TOWFloatSinkPin.SendFloatData ( Data : Single );
begin
  if ( Assigned( SendFloatEvent )) then
    SendFloatEvent( Data, FCustomData );
end;
//---------------------------------------------------------------------------
constructor TOWIntegerSourcePin.Create( AOwner: TComponent; APinNotificationEvent : TOWIntegerPinNotificationEvent );
begin
  inherited Create( AOwner );
  PinNotificationEvent := APinNotificationEvent;

  AddType( IOWIntegerStream, Notification );
end;
//---------------------------------------------------------------------------
function TOWIntegerSourcePin.Notification( Handler : IOWStream; Operation : Integer; Data : Pointer; State : TOWNotificationState ) : TOWNotificationResults;
var
  Interf : IOWIntegerStream;

begin
  if( Assigned( PinNotificationEvent )) then
    if( Handler.QueryInterface(IOWIntegerStream,Interf) = 0 ) then
       begin
       Result := PinNotificationEvent( Interf, Operation, Data, State );
       Exit;
       end;

  Result := [];
end;
//---------------------------------------------------------------------------
constructor TOWIntegerSinkPin.Create( AOwner: TComponent; ASendIntegerEvent : TOWSendIntegerEvent; ACustomData : Pointer );
begin
  inherited Create( AOwner );
  SendIntegerEvent := ASendIntegerEvent;
  FCustomData := ACustomData;
end;
//---------------------------------------------------------------------------
procedure TOWIntegerSinkPin.SendIntegerData ( Data : Integer );
begin
  if ( Assigned( SendIntegerEvent )) then
    SendIntegerEvent( Data, FCustomData );
end;
//---------------------------------------------------------------------------
constructor TOWRealSourcePin.Create( AOwner: TComponent; APinNotificationEvent : TOWRealPinNotificationEvent );
begin
  inherited Create( AOwner );
  PinNotificationEvent := APinNotificationEvent;

  AddType( IOWRealStream, Notification );
end;
//---------------------------------------------------------------------------
function TOWRealSourcePin.Notification( Handler : IOWStream; Operation : Integer; Data : Pointer; State : TOWNotificationState ) : TOWNotificationResults;
var
  Interf : IOWRealStream;

begin
  if( Assigned( PinNotificationEvent )) then
    if( Handler.QueryInterface(IOWRealStream,Interf) = 0 ) then
       begin
       Result := PinNotificationEvent( Interf, Operation, Data, State );
       Exit;
       end;

  Result := [];
end;
//---------------------------------------------------------------------------
constructor TOWRealSinkPin.Create( AOwner: TComponent; ASendRealEvent : TOWSendRealEvent; ACustomData : Pointer );
begin
  inherited Create( AOwner );
  SendRealEvent := ASendRealEvent;
  FCustomData := ACustomData;
end;
//---------------------------------------------------------------------------
procedure TOWRealSinkPin.SendRealData ( Data : Real );
begin
  if ( Assigned( SendRealEvent )) then
    SendRealEvent( Data, FCustomData );
end;
//---------------------------------------------------------------------------


procedure Register;
begin
  OWRegisterWireStream( IOWIntegerStream, 'Integer single' );
  OWRegisterWireStream( IOWFloatStream,   'Float single' );
  OWRegisterWireStream( IOWRealStream,    'Real single' ); 
end;
end.
